
#include "histogram.h"
#include "histogram_internal.h"
#include <vector>

using namespace std;

bool find_minmax(const vector<double>& numbers, double& min, double& max) {
    if (numbers.empty())
        return false;
    min = numbers[0];
    max = numbers[0];
    for (double x : numbers) {
        if (x < min)
            min = x;
        if (x > max)
            max = x;
    }
    return true;
}

vector<size_t> make_histogram(const vector<double>& numbers, size_t bin_count) {
    vector<size_t> bins(bin_count, 0);
    double min, max;
    find_minmax(numbers, min, max);
    double bin_size = (max - min) / bin_count;

    for (size_t i = 0; i < numbers.size(); i++) {
        bool found = false;
        for (size_t j = 0; j < bin_count - 1 && !found; j++) {
            double lo = min + j * bin_size;
            double hi = min + (j + 1) * bin_size;
            if (numbers[i] >= lo && numbers[i] < hi) {
                bins[j]++;
                found = true;
            }
        }
        if (!found)
            bins[bin_count - 1]++;
    }
    return bins;
}

double compute_bin_size(const std::vector<double>& numbers, size_t bin_count) {
    double min, max;
    if (!find_minmax(numbers, min, max))
        return 0.0;
    return (max - min) / bin_count;
}
